

set -ex



ruby --version
gem --version
ruby -e 'puts "Hi, from Ruby!"'
ruby -r rubygems -e 'puts "Has gems."; exit 0'
ruby -r fiddle -r openssl -r yaml -r zlib -e 'puts "Has all dependencies."; exit 0'
ruby -r readline -e 'puts "Has readline."; exit 0'
cd native
ruby config.rb
make
ruby -r ./hello.so -e 'hello'
exit 0
