module Test
  module Unit
    module UI
      module Console
        module OutputLevel
          SILENT = 0
          PROGRESS_ONLY = 1
          IMPORTANT_FAULTS_ONLY = 2
          NORMAL = 3
          VERBOSE = 4
        end
      end
    end
  end
end
