#ifndef RBIMPL_ARITHMETIC_SHORT_H                    /*-*-C++-*-vi:se ft=cpp:*/
#define RBIMPL_ARITHMETIC_SHORT_H
/**
 * @file
 * @author     Ruby developers <ruby-core@ruby-lang.org>
 * @copyright  This  file  is   a  part  of  the   programming  language  Ruby.
 *             Permission  is hereby  granted,  to  either redistribute  and/or
 *             modify this file, provided that  the conditions mentioned in the
 *             file COPYING are met.  Consult the file for details.
 * @warning    Symbols   prefixed  with   either  `RBIMPL`   or  `rbimpl`   are
 *             implementation details.   Don't take  them as canon.  They could
 *             rapidly appear then vanish.  The name (path) of this header file
 *             is also an  implementation detail.  Do not expect  it to persist
 *             at the place it is now.  Developers are free to move it anywhere
 *             anytime at will.
 * @note       To  ruby-core:  remember  that   this  header  can  be  possibly
 *             recursively included  from extension  libraries written  in C++.
 *             Do not  expect for  instance `__VA_ARGS__` is  always available.
 *             We assume C99  for ruby itself but we don't  assume languages of
 *             extension libraries.  They could be written in C++98.
 * @brief      Arithmetic conversion between C's `short` and Ruby's.
 *
 * Shyouhei  wonders:  why  there  is   no  SHORT2NUM,  given  there  are  both
 * #USHORT2NUM and #CHR2FIX?
 */
#include "ruby/internal/value.h"
#include "ruby/internal/dllexport.h"
#include "ruby/internal/special_consts.h"

#define RB_NUM2SHORT  rb_num2short_inline /**< @alias{rb_num2short_inline} */
#define RB_NUM2USHORT rb_num2ushort       /**< @alias{rb_num2ushort} */
#define NUM2SHORT     RB_NUM2SHORT        /**< @old{RB_NUM2SHORT} */
#define NUM2USHORT    RB_NUM2USHORT       /**< @old{RB_NUM2USHORT} */
#define USHORT2NUM    RB_INT2FIX          /**< @old{RB_INT2FIX} */
#define RB_FIX2SHORT  rb_fix2short        /**< @alias{rb_fix2ushort} */
#define FIX2SHORT     RB_FIX2SHORT        /**< @old{RB_FIX2SHORT} */

RBIMPL_SYMBOL_EXPORT_BEGIN()

/**
 * Converts an instance of ::rb_cNumeric into C's `short`.
 *
 * @param[in]  num             Something numeric.
 * @exception  rb_eTypeError   `num` is not a numeric.
 * @exception  rb_eRangeError  `num` is out of range of `short`.
 * @return     The passed value converted into C's `short`.
 */
short rb_num2short(VALUE num);

/**
 * Converts an instance of ::rb_cNumeric into C's `unsigned short`.
 *
 * @param[in]  num             Something numeric.
 * @exception  rb_eTypeError   `num` is not a numeric.
 * @exception  rb_eRangeError  `num` is out of range of `unsigned short`.
 * @return     The passed value converted into C's `unsigned short`.
 */
unsigned short rb_num2ushort(VALUE num);

/**
 * Identical to rb_num2short().
 *
 * @param[in]  num             Something numeric.
 * @exception  rb_eTypeError   `num` is not a numeric.
 * @exception  rb_eRangeError  `num` is out of range of `short`.
 * @return     The passed value converted into C's `short`.
 *
 * @internal
 *
 * This function seems to be a complete  waste of disk space.  @shyouhei has no
 * idea why this is a different thing from rb_num2short().
 */
short rb_fix2short(VALUE num);

/**
 * Identical to rb_num2ushort().
 *
 * @param[in]  num             Something numeric.
 * @exception  rb_eTypeError   `num` is not a numeric.
 * @exception  rb_eRangeError  `num` is out of range of `unsigned short`.
 * @return     The passed value converted into C's `unsigned short`.
 *
 * @internal
 *
 * This function seems to be a complete  waste of disk space.  @shyouhei has no
 * idea why this is a different thing from rb_num2ushort().
 */
unsigned short rb_fix2ushort(VALUE num);
RBIMPL_SYMBOL_EXPORT_END()

/**
 * Identical to rb_num2short().
 *
 * @param[in]  x               Something numeric.
 * @exception  rb_eTypeError   `x` is not a numeric.
 * @exception  rb_eRangeError  `x` is out of range of `short`.
 * @return     The passed value converted into C's `short`.
 *
 * @internal
 *
 * This function seems to  be a complete waste of time.   @shyouhei has no idea
 * why this is a different thing from rb_num2short().
 */
static inline short
rb_num2short_inline(VALUE x)
{
    if (RB_FIXNUM_P(x))
        return rb_fix2short(x);
    else
        return rb_num2short(x);
}

#endif /* RBIMPL_ARITHMETIC_SHORT_H */
